import { DatePipe } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { ColDef } from 'ag-grid-community';
import { NotificationService } from '../../../services/notification.service';
import { StructureAdminInfo } from '../../models/demandAttachment.model';
import { AdminService } from '../../services/admin.service';

@Component({
  selector: 'app-admin-claim-structure',
  templateUrl: './claim-structure.component.html',
  styleUrls: ['../../admin.scss'],
})
export class ClaimStructureComponent implements OnInit {
  constructor(
    private adminService: AdminService,
    private notificationService: NotificationService,
    private datePipe: DatePipe
  ) {}
  public demandsAttachment: StructureAdminInfo[];

  public columnsDefs: ColDef<StructureAdminInfo>[] = [
    {
      headerName: 'Utilisateur',
      field: 'userEmail',
    },
    {
      headerName: 'Structure',
      field: 'structureName',
      flex: 1,
    },
    {
      headerName: 'Date de demande',
      field: 'createdAt',
      valueFormatter: (params): string => this.datePipe.transform(params.value, 'mediumDate'),
    },
    {
      headerName: 'Valider',
      width: 100,
      cellRenderer: (): string => '<button type="button">Valider</button>',
      onCellClicked: (params): void => {
        this.acceptDemand(params.data);
      },
    },
    {
      headerName: 'Refuser',
      width: 100,
      cellRenderer: (): string => '<button type="button">Refuser</button>',
      onCellClicked: (params): void => {
        this.refuseDemand(params.data);
      },
    },
  ];

  ngOnInit(): void {
    this.adminService.getPendingStructure().subscribe((demands) => {
      this.demandsAttachment = demands;
    });
  }

  public acceptDemand(demand: StructureAdminInfo): void {
    this.adminService.acceptStructureClaim(demand.userEmail, demand.structureId, demand.structureName).subscribe({
      next: (data) => {
        this.demandsAttachment = data;
        this.notificationService.showSuccess('Demande acceptée avec succès');
      },
      error: () => {
        this.notificationService.showError('Une erreur est survenue');
      },
    });
  }

  public refuseDemand(demand: StructureAdminInfo): void {
    this.adminService.refuseStructureClaim(demand.userEmail, demand.structureId, demand.structureName).subscribe({
      next: (data) => {
        this.demandsAttachment = data;
        this.notificationService.showSuccess('Demande refusée avec succès');
      },
      error: () => {
        this.notificationService.showError('Une erreur est survenue');
      },
    });
  }
}
