import { DatePipe } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { ColDef } from 'ag-grid-community';
import { finalize } from 'rxjs/operators';
import { Structure } from '../../../models/structure.model';
import { AdminService } from '../../services/admin.service';

@Component({
  selector: 'app-deleted-structures',
  templateUrl: './deleted-structures.component.html',
  styleUrls: ['../../admin.scss'],
})
export class DeletedStructuresComponent implements OnInit {
  constructor(private adminService: AdminService, private datePipe: DatePipe) {}
  public deletedStructures: Structure[];
  public selectedStructure: Structure;
  public isLoading = true;

  public columnDefs: ColDef<Structure>[] = [
    {
      headerName: 'Structure',
      field: 'structureName',
      flex: 1,
      sortable: true,
      cellRenderer: (params): string => this.renderLink(params.data),
    },
    {
      headerName: 'Date de création',
      field: 'createdAt',
      width: 190,
      sortable: true,
      valueFormatter: (params): string => this.datePipe.transform(params.value, 'yyyy-MM-dd'),
    },
    {
      headerName: 'Date de suppression',
      field: 'deletedAt',
      width: 190,
      sortable: true,
      sort: 'desc',
      valueFormatter: (params): string => this.datePipe.transform(params.value, 'yyyy-MM-dd'),
    },
    {
      headerName: 'Restaurer',
      cellRenderer: (): string => '<button type="button">Restaurer</button>',
      onCellClicked: (params): void => {
        this.restoreStructure(params.data._id);
      },
    },
  ];

  ngOnInit(): void {
    this.loadDeletedStructures();
  }

  private loadDeletedStructures(): void {
    this.adminService
      .getDeletedStructures()
      .pipe(finalize(() => (this.isLoading = false)))
      .subscribe((structures) => {
        this.deletedStructures = structures;
      });
  }

  private renderLink(structure: Structure): string {
    return `<a href="/acteurs?id=${structure._id}" target="_blank">${structure.structureName}</a>`;
  }

  private restoreStructure(structureId: string): void {
    this.adminService.restoreDeletedStructure(structureId).subscribe(() => {
      this.isLoading = true;
      this.loadDeletedStructures();
    });
  }
}
