import { Component, OnInit } from '@angular/core';
import { ColDef } from 'ag-grid-community';
import { User } from '../../../models/user.model';
import { AdminService } from '../../services/admin.service';
import { EspaceCoopCNFS } from './../../../models/espaceCoopCNFS.model';

@Component({
  selector: 'app-espace-coop-cnfs',
  templateUrl: './espace-coop-cnfs.component.html',
  styleUrls: ['../../admin.scss'],
})
export class EspaceCoopCNFSComponent implements OnInit {
  public unregisteredCNFS: EspaceCoopCNFS[] = [];
  public unregisteredColumnDefs: ColDef<EspaceCoopCNFS>[] = [
    {
      headerName: 'Nom',
      field: 'name',
      sortable: true,
      sort: 'asc',
      resizable: true,
    },
    {
      headerName: 'Employeur',
      field: 'employer',
      sortable: true,
      // Case insensitive alphabetical order
      comparator: (a, b): number => a.toLowerCase().localeCompare(b.toLowerCase()),
      flex: 1,
      resizable: true,
    },
    {
      headerName: 'Structures',
      field: 'structures',
      cellRenderer: (param): string => param.data.structures.join('<br/>'),
      flex: 1,
      resizable: true,
    },
    {
      headerName: 'Email',
      field: 'email',
      flex: 1,
      resizable: true,
    },
    {
      headerName: 'Téléphone',
      field: 'phone',
      resizable: true,
    },
  ];

  public cnfsNotInEspaceCoop: User[] = [];
  public notInEspaceCoopColumnDefs: ColDef<User>[] = [
    {
      headerName: 'Prénom',
      field: 'name',
      sortable: true,
    },
    {
      headerName: 'Nom',
      field: 'surname',
      sort: 'asc',
    },
    {
      headerName: 'Email',
      field: 'email',
    },
    {
      headerName: 'Téléphone',
      field: 'phone',
    },
  ];

  public getRowHeight(params: { data: EspaceCoopCNFS }): number {
    const structures = params.data.structures || [];
    if (structures.length <= 1) return 40;
    return structures.length * 40;
  }

  constructor(private adminService: AdminService) {}
  ngOnInit(): void {
    this.adminService.getAllEspaceCoopCNFS().subscribe((espaceCoopCNFS) => {
      this.unregisteredCNFS = espaceCoopCNFS.filter((member) => !member.hasAccount);
      this.adminService.getAllResinCNFS().subscribe((resinCNFS) => {
        this.cnfsNotInEspaceCoop = resinCNFS.filter(
          (resin) => !espaceCoopCNFS.some((coop) => coop.email === resin.email)
        );
      });
    });
  }
}
