import { DatePipe } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { ColDef } from 'ag-grid-community';
import { finalize } from 'rxjs/operators';
import { AdminStructure } from '../../models/adminStructureList.interface';
import { AdminService } from '../../services/admin.service';

@Component({
  selector: 'app-admin-structures-list',
  templateUrl: './admin-structures-list.component.html',
  styleUrls: ['../../admin.scss'],
})
export class AdminStructuresListComponent implements OnInit {
  constructor(private adminService: AdminService, private datePipe: DatePipe, private router: Router) {}
  public structuresClaimed: AdminStructure[];
  public structuresInClaim: AdminStructure[];
  public structuresToClaim: AdminStructure[];
  public structuresIncomplete: AdminStructure[];
  public isAll = false;
  public isLoading = true;
  public rowHeight = 25;

  public columnDefs: ColDef<AdminStructure>[] = [
    {
      headerName: 'Structure',
      cellRenderer: (params): string => this.renderLink(params.data.structureId, params.data.structureName),
      comparator(_, __, nodeA, nodeB): number {
        return nodeA.data.structureName.toLowerCase() > nodeB.data.structureName.toLocaleLowerCase() ? -1 : 1;
      },
      sortable: true,
      flex: 3,
    },
    {
      headerName: 'Date de mise à jour',
      field: 'updatedAt',
      valueFormatter: ({ value }): string => this.datePipe.transform(value, 'mediumDate'),
      cellClass: (params): string => {
        return params.data.isOutdated ? 'red' : '';
      },
      flex: 1,
      sortable: true,
    },
  ];

  ngOnInit(): void {
    this.adminService
      .getAllStructureAdmin()
      .pipe(finalize(() => (this.isLoading = false)))
      .subscribe((response) => {
        this.structuresClaimed = response.claimed;
        this.structuresInClaim = response.inClaim;
        this.structuresToClaim = response.toClaim;
        this.structuresIncomplete = response.incomplete;
      });
  }

  private renderLink(structureId: string, structureName: string): string {
    return `<a href="/acteurs?id=${structureId}" target="_blank">${structureName}</a>`;
  }
}
