import { Component, OnInit } from '@angular/core';
import { UserAnnuary } from '../models/user.model';
import { AuthService } from '../services/auth.service';
import { SearchService } from '../structure-list/services/search.service';
import { SearchQuery } from './models/searchQuery.model';

@Component({
  selector: 'app-annuaire',
  templateUrl: './annuaire.component.html',
  styleUrls: ['./annuaire.component.scss'],
})
export class AnnuaireComponent implements OnInit {
  constructor(private searchService: SearchService, private authService: AuthService) {}
  public userList: UserAnnuary[] = [];
  public totalUserResult: number;
  public resetFilters = 0;
  public nextPage = 1;
  public isLoading = true;

  ngOnInit(): void {
    if (!this.userIsLoggedIn()) {
      this.searchService.getUserRegistryCount().subscribe((count) => {
        this.totalUserResult = count;
      });
      this.isLoading = false;
    }

    this.userList = this.searchService.previousResult$.getValue().docs;
    this.totalUserResult = this.searchService.previousResult$.getValue().count;
    if (this.userList.length !== 0) {
      this.isLoading = false;
    }
  }

  public userIsLoggedIn(): boolean {
    return this.authService.isLoggedIn();
  }

  private loadParams(params: SearchQuery): void {
    this.searchService.annuaireSearchQuery = params;
  }

  public searchUsers(params: SearchQuery): void {
    this.loadParams(params);
    this.searchService
      .searchUserRegistry(params.search, params.page, params.jobFilters, params.employerFilters)
      .then((res) => {
        this.searchService.previousResult$.next({ count: res.count, docs: res.docs });
        this.userList = res.docs;
        this.totalUserResult = res.count;
        this.isLoading = false;
      });
  }

  public shouldResetFilters(): void {
    this.resetFilters++;
  }
  public showMore(): void {
    this.nextPage++;
  }

  /**
   * Should load in specific conditions such as changes in filters and search input.
   * Should NOT load when showing more result because it would result in losing scroll position because of the rerender of the list.
   */
  public shouldLoad(value: boolean): void {
    this.isLoading = value;
  }
}
