import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { ButtonTypeV3 } from '../../shared/components/v3/button/button-type.enum';
import { TypeModal } from '../enums/TypeModal.enum';

@Component({
  selector: 'app-filter-modal',
  templateUrl: './filter-modal.component.html',
  styleUrls: ['./filter-modal.component.scss'],
})
export class FilterModalComponent implements OnInit, OnChanges {
  @Input() public modalType: TypeModal;
  @Input() public filtersTypes: string[];
  @Input() public checkedFilters: string[];
  @Input() public jobFilterChecked: string[];
  @Input() public employerFilterChecked: string[];
  @Output() fetchResults = new EventEmitter<string[]>();
  public buttonTypeV3Enum = ButtonTypeV3;
  public toggledCategories: string[] = [];
  public currentCheckedFilters: string[] = [];

  ngOnInit(): void {
    this.currentCheckedFilters = this.checkedFilters.slice();
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.modalType) {
      this.currentCheckedFilters = this.checkedFilters.slice();
    }
  }

  public isFilterChecked(filter: string): boolean {
    return this.currentCheckedFilters.includes(filter);
  }

  public toggleCheckbox(filter: string): void {
    const filterExists = this.currentCheckedFilters.includes(filter);

    if (!filterExists) {
      const index = this.filtersTypes.indexOf(filter);
      this.currentCheckedFilters.push(this.filtersTypes[index]);
    } else {
      const filterToRemove = this.currentCheckedFilters.indexOf(filter);
      this.currentCheckedFilters.splice(filterToRemove, 1);
    }
  }

  public getModalType(): string {
    switch (this.modalType) {
      case TypeModal.employers:
        return 'employers';
      case TypeModal.jobs:
        return 'jobs';
      default:
        return '';
    }
  }

  /** Clear only filters in the current modal */
  public clearFilters(): void {
    if (this.modalType === TypeModal.jobs) {
      this.currentCheckedFilters = this.employerFilterChecked;
    } else if (this.modalType === TypeModal.employers) {
      this.currentCheckedFilters = this.jobFilterChecked;
    }

    this.fetchResults.emit(this.currentCheckedFilters);
  }

  public onSubmitFilters(): void {
    this.fetchResults.emit(this.currentCheckedFilters);
  }
}
