import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { AbstractControl, UntypedFormGroup } from '@angular/forms';
import { ButtonTypeV3 } from '../../../../shared/components/v3/button/button-type.enum';
import { Module } from '../../../../structure-list/models/module.model';

@Component({
  selector: 'app-structure-equipments',
  templateUrl: './structure-equipments.component.html',
  styleUrls: ['./structure-equipments.component.scss'],
})
export class StructureEquipmentsComponent implements OnInit {
  @Input() structureForm: UntypedFormGroup;
  @Input() isEditMode: boolean;
  @Input() selfServiceMaterial: {
    module: Module;
    openned: boolean;
  }[];
  @Output() validateForm = new EventEmitter<any>();

  public buttonTypeEnumV3 = ButtonTypeV3;

  private vowels = ['a', 'e', 'i', 'o', 'u', 'y'];

  async ngOnInit(): Promise<void> {
    this.validateForm.emit();
  }

  public getStructureControl(nameControl: string): AbstractControl {
    return this.structureForm.get(nameControl);
  }

  public changeValueHandler(equipment: string, value = 0): void {
    let field = '';
    if (equipment === 'scanner') field = 'nbScanners';
    if (equipment === 'computer') field = 'nbComputers';
    if (equipment === 'printer') field = 'nbPrinters';

    if (value === -1 && this.structureForm.value[field] === 0) return;
    if (this.structureForm.value[field] + value < 0) return;
    this.getStructureControl(field).setValue(Math.round(this.structureForm.value[field] + value));
    this.validateForm.emit();
  }

  public setValidationsForm(): void {
    this.validateForm.emit();
  }

  /**
   * Check if first letter is a vowel and returns text with correct preposition
   * @param equipment Equipment
   * @returns "de tablette(s)" | "d'imprimante(s)"
   */
  public formatEquipment(equipment: string): string {
    if (this.vowels.includes(equipment.toLocaleLowerCase()[0])) {
      return `d'${equipment.toLocaleLowerCase()}s`;
    } else {
      return `de ${equipment.toLocaleLowerCase()}s`;
    }
  }
  public goBack(): void {
    history.back();
  }
}
