import { Component, Input, OnInit } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { OrientationService } from '../../../../services/orientation.service';
import { PersonalOfferService } from '../../../../services/personal-offer.service';
import { CategoryEnum } from '../../../../shared/enum/category.enum';
import { Category } from '../../../../structure-list/models/category.model';
import { Module } from '../../../../structure-list/models/module.model';
import { SearchService } from '../../../../structure-list/services/search.service';
import { OrientationUtils } from '../../../../utils/orientationUtils';
import { SelectComponent } from '../../global-components/select/select.component';
import { FiltersForm } from '../../interfaces/filtersForm.interface';

@Component({
  selector: 'app-base-skills-choice',
  templateUrl: './base-skills-choice.component.html',
})
export class BaseSkillsChoiceComponent extends SelectComponent implements OnInit {
  @Input() form: FormGroup<FiltersForm>;
  public orientationUtils = new OrientationUtils();
  public baseSkills: Module[] = [];

  constructor(
    private searchService: SearchService,
    public orientationService: OrientationService,
    public personalOfferService: PersonalOfferService
  ) {
    super();
  }

  ngOnInit(): void {
    this.selectedModules = this.form.get('filters').value;
    this.searchService.getCategories().subscribe(async (categories: Category[]) => {
      this.baseSkills = categories.find((el) => el.id === CategoryEnum.baseSkills).modules;

      // For rdv from structure or annuaire, disable items which are not offered
      if (this.orientationService.rdvUser || this.orientationService.rdvStructure) {
        await this.orientationUtils.rdvDisableItemsNotOffered(this.baseSkills, this.orientationService, 'baseSkills');
      }
    });

    this.checkValidation.emit();
  }
}
