import { Component, EventEmitter, Input, Output } from '@angular/core';
import { FormGroup } from '@angular/forms';

import { OrientationUtils } from '../../../utils/orientationUtils';
import { FiltersSteps } from '../enums/filtersSteps.enum';
import { GenericOrientationSteps } from '../enums/genericOrientationSteps.enum';
import { RecapsType } from '../enums/recapsType.enum';
import { StructuresListSteps } from '../enums/structuresListSteps.enum';
import { FiltersForm } from '../interfaces/filtersForm.interface';
import { MediationType } from '../types/mediation.type';
import { MediationStepType } from '../types/mediationStep.type';

@Component({
  selector: 'app-base-skills',
  templateUrl: './base-skills.component.html',
})
export class BaseSkillsComponent {
  @Input() currentStep: FiltersSteps | MediationStepType;
  @Input() currentType: GenericOrientationSteps | MediationType;
  @Input() form: FormGroup<FiltersForm>;
  @Output() validatePage = new EventEmitter<any>();

  public orientationUtils = new OrientationUtils();
  public pagesValidation: any[] = [];

  // Enums
  public structuresListStepsEnum = StructuresListSteps;
  public FiltersSteps = FiltersSteps;
  public recapsType = RecapsType;
  public GenericOrientationSteps = GenericOrientationSteps;

  public checkValidation(): void {
    switch (this.currentType) {
      case GenericOrientationSteps.common:
        this.orientationUtils.setValidationsEquipmentForm(
          this.pagesValidation,
          this.form,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as FiltersSteps
        );
        break;
      default:
        throw new Error('Not implemented tunnel type in BaseSkillsComponent');
    }
  }
}
