import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Observable, forkJoin, tap } from 'rxjs';
import { PersonalOffer } from '../../../../models/personalOffer.model';
import { OrientationService } from '../../../../services/orientation.service';
import { PersonalOfferService } from '../../../../services/personal-offer.service';
import { INeedItem, OrientationUtils } from '../../../../utils/orientationUtils';
import { NeedsType } from '../../enums/needs.enum';

@Component({
  selector: 'app-needs-selection',
  templateUrl: './needs-selection.component.html',
})
export class NeedsSelectionComponent implements OnInit {
  @Input() currentNeed: NeedsType;
  @Output() setNeedType = new EventEmitter<any>();
  @Output() validate = new EventEmitter<any>();
  public needsList: INeedItem[] = new OrientationUtils().needsList;

  constructor(public orientationService: OrientationService, private personalOfferService: PersonalOfferService) {}

  ngOnInit(): void {
    if (this.orientationService.rdvUser || this.orientationService.rdvStructure) {
      this.needsList = this.needsList.filter(
        (item: INeedItem) => item.key === NeedsType.onlineDemarch || item.key === NeedsType.learnSkills
      );
    }

    if (this.orientationService.rdvUser) {
      // Get the user personalOffers in this screen so they will be available to disable items in the next screen (cf. function rdvDisableItemsNotOffered)
      if (this.orientationService.rdvUser.personalOffers?.length) {
        const personalOffers$: Observable<any>[] = [];
        this.orientationService.rdvUser.personalOffers.forEach((personalOffer) => {
          personalOffers$.push(
            this.personalOfferService.getPersonalOffer(personalOffer).pipe(
              tap((personalOffer) => {
                this.orientationService.rdvUser.personalOffersPopulated.push(new PersonalOffer(personalOffer));
              })
            )
          );
        });
        forkJoin(personalOffers$).subscribe();
      }
    }
  }

  public selectNeed(event: NeedsType): void {
    this.setNeedType.emit(event);
  }
  public getSelected(): string | NeedsType {
    const selected = this.needsList.filter((need) => need.key === this.currentNeed)[0];
    if (selected) {
      this.validate.emit();
      return selected.key;
    }
  }
}
