import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { Owner } from '../../../../../models/owner.model';
import { Structure } from '../../../../../models/structure.model';
import { NotificationService } from '../../../../../services/notification.service';
import { OrientationService } from '../../../../../services/orientation.service';

@Component({
  selector: 'app-appointment-end',
  templateUrl: './appointment-end.component.html',
  styleUrls: ['./appointment-end.component.scss'],
})
export class AppointmentEndComponent implements OnInit {
  @Input() form: UntypedFormGroup;
  @Input() structureRDV: Structure;
  @Input() socialWorker: Owner;
  @Input() selectedStructureRDV: Structure;
  @Output() setResetOrientation = new EventEmitter();
  @Output() checkValidation = new EventEmitter<any>();

  public hasStructure = false;

  constructor(private notificationService: NotificationService, public orientationService: OrientationService) {}

  async ngOnInit(): Promise<void> {
    this.checkValidation.emit();
    if (this.form.get('structureRDV').value || this.structureRDV) this.hasStructure = true;
    try {
      if (!this.structureRDV) {
        this.setResetOrientation.emit();
      }
    } catch (error) {
      console.error(error);
      this.notificationService.showError('Une erreur est survenue', error.message);
    }
  }
}
