import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { CategoryEnum } from 'src/app/shared/enum/category.enum';
import { OrientationService } from '../../../../services/orientation.service';
import { PersonalOfferService } from '../../../../services/personal-offer.service';
import { StructureService } from '../../../../services/structure.service';
import { ButtonType } from '../../../../shared/components/button/buttonType.enum';
import { Category } from '../../../../structure-list/models/category.model';
import { Module } from '../../../../structure-list/models/module.model';
import { SearchService } from '../../../../structure-list/services/search.service';
import { OrientationUtils } from '../../../../utils/orientationUtils';

@Component({
  selector: 'app-online-demarch',
  templateUrl: './online-demarch.component.html',
})
export class OnlineDemarchComponent implements OnInit {
  @Input() form: UntypedFormGroup;
  @Output() checkValidation = new EventEmitter<any>();

  constructor(
    private searchService: SearchService,
    public structureService: StructureService,
    public orientationService: OrientationService,
    public personalOfferService: PersonalOfferService
  ) {}
  public orientationUtils = new OrientationUtils();
  public accompanimentType: Module[];
  public buttonTypeEnum = ButtonType;
  public selectedModules: Module[] = [];
  public showStrangersModal = false;

  ngOnInit(): void {
    this.selectedModules = this.form.get('onlineDemarcheType').value;
    // TODO move this in form init categories
    this.searchService.getCategories().subscribe(async (categories: Category[]) => {
      this.accompanimentType = categories.find((el) => el.id === CategoryEnum.onlineProcedures).modules;

      // For rdv from structure or annuaire, disable items which are not offered
      if (this.orientationService.rdvUser || this.orientationService.rdvStructure) {
        await this.orientationUtils.rdvDisableItemsNotOffered(
          this.accompanimentType,
          this.orientationService,
          'onlineProcedures'
        );
      }
    });

    this.checkValidation.emit();
  }

  public handleClick(module: Module): void {
    if (module.id === 'foreigners' && !this.isSelectedModule(module.id)) {
      this.showStrangersModal = true;
    }
    if (this.isSelectedModule(module.id)) {
      const index = this.selectedModules.findIndex((_module) => _module.id === module.id);
      this.selectedModules.splice(index, 1);
    } else {
      this.selectedModules.push(module);
    }
    this.form.get('onlineDemarcheType').patchValue(this.selectedModules);
    this.checkValidation.emit();
  }

  public isSelectedModule(moduleId: string): boolean {
    if (this.selectedModules?.map((module) => module.id).includes(moduleId)) return true;
    return false;
  }
  public handleClose(): void {
    this.showStrangersModal = false;
  }
}
