import { Day } from './day.model';

type dayType = 'monday' | 'tuesday' | 'wednesday' | 'thursday' | 'friday' | 'saturday' | 'sunday';

export class Week {
  monday: Day;
  tuesday: Day;
  wednesday: Day;
  thursday: Day;
  friday: Day;
  saturday: Day;
  sunday: Day;

  constructor(obj?: any) {
    Object.assign(this, obj, {
      monday: obj?.monday ? new Day(obj.monday) : new Day(),
      tuesday: obj?.tuesday ? new Day(obj.tuesday) : new Day(),
      wednesday: obj?.wednesday ? new Day(obj.wednesday) : new Day(),
      thursday: obj?.thursday ? new Day(obj.thursday) : new Day(),
      friday: obj?.friday ? new Day(obj.friday) : new Day(),
      saturday: obj?.saturday ? new Day(obj.saturday) : new Day(),
      sunday: obj?.sunday ? new Day(obj.sunday) : new Day(),
    });
  }

  /**
   * When iterating over Days in a week, override the default "keyvalue" pipe which gives incorrect typing.
   * @example
   * for (let day of structure.hours)...
   * -> day.key = tuesday
   * -> day.value = {open: true, time: [...]}
   * @yields day of the week
   */
  *[Symbol.iterator](): Generator<{ key: dayType; value: Day }> {
    yield { key: 'monday', value: this.monday };
    yield { key: 'tuesday', value: this.tuesday };
    yield { key: 'wednesday', value: this.wednesday };
    yield { key: 'thursday', value: this.thursday };
    yield { key: 'friday', value: this.friday };
    yield { key: 'saturday', value: this.saturday };
    yield { key: 'sunday', value: this.sunday };
  }

  public getDayTranslation(day: dayType): string {
    switch (day) {
      case 'monday':
        return 'lundi';
      case 'tuesday':
        return 'mardi';
      case 'thursday':
        return 'jeudi';
      case 'wednesday':
        return 'mercredi';
      case 'friday':
        return 'vendredi';
      case 'saturday':
        return 'samedi';
      case 'sunday':
        return 'dimanche';
      default:
        return null;
    }
  }

  public hasData(): boolean {
    if (
      this.monday.time.length === 0 &&
      this.tuesday.time.length === 0 &&
      this.wednesday.time.length === 0 &&
      this.thursday.time.length === 0 &&
      this.friday.time.length === 0 &&
      this.saturday.time.length === 0 &&
      this.sunday.time.length === 0
    ) {
      return false;
    }
    return true;
  }
}
