import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { AbstractControl, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { lastValueFrom } from 'rxjs';
import { StructureWithOwners } from '../../models/structureWithOwners.model';
import { TempUser } from '../../models/temp-user.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { CustomRegExp } from '../../utils/CustomRegExp';

@Component({
  selector: 'app-structure-add-member-modal',
  templateUrl: './structure-add-member-modal.component.html',
  styleUrls: ['./structure-add-member-modal.component.scss'],
})
export class StructureAddMemberModalComponent implements OnInit {
  @Input() public opened: boolean;
  @Input() public structure: StructureWithOwners;
  @Output() closed = new EventEmitter<boolean>();
  public formAddAccount: FormGroup;

  constructor(
    private formBuilder: FormBuilder,
    private structureService: StructureService,
    private notificationService: NotificationService
  ) {}

  ngOnInit(): void {
    this.formAddAccount = this.formBuilder.group({
      email: ['', [Validators.required, Validators.pattern(CustomRegExp.EMAIL)]],
    });
  }
  public async closeModal(memberAddRequested: boolean): Promise<void> {
    if (memberAddRequested) {
      await this.addOwner();
    }
    this.closed.emit(memberAddRequested);
  }

  // getter for form fields
  get fAddAccount(): Record<string, AbstractControl> {
    return this.formAddAccount.controls;
  }

  public async addOwner(): Promise<void> {
    // stop here if form is invalid
    if (this.formAddAccount.invalid) {
      return;
    }
    const user = new TempUser();
    user.email = this.fAddAccount.email.value;

    try {
      const res = await lastValueFrom(this.structureService.addOwnerToStructure(user, this.structure.structure._id));
      if ((res as TempUser).email) {
        this.notificationService.showSuccess("La demande d'ajout a bien été effectuée");
      } else {
        this.notificationService.showSuccess('Le membre a bien été ajouté');
      }
      this.fAddAccount.email.setValue('');
    } catch (e) {
      if (e.status === 422) {
        this.notificationService.showError('', `L'email ${user.email} est déjà rattaché à la structure`);
      } else {
        this.notificationService.showErrorPleaseRetry("Le membre n'a pas pu être ajouté");
      }
    }
  }
}
