import { Component, OnInit } from '@angular/core';
import { AbstractControl, UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthService } from '../services/auth.service';
import { NotificationService } from '../services/notification.service';
import { ButtonTypeV3 } from '../shared/components/v3/button/button-type.enum';

@Component({
  selector: 'app-reset-password',
  templateUrl: './reset-password.component.html',
  styleUrls: ['./reset-password.component.scss'],
})
export class ResetPasswordComponent implements OnInit {
  public resetForm: UntypedFormGroup;
  public loading = false;
  public submitted = false;
  public token: string;
  // Condition form
  public isShowConfirmPassword = false;
  public isShowPassword = false;
  public buttonTypeEnumV3 = ButtonTypeV3;

  constructor(
    private formBuilder: UntypedFormBuilder,
    private authService: AuthService,
    private router: Router,
    private notificationService: NotificationService,
    private activatedRoute: ActivatedRoute
  ) {}

  ngOnInit(): void {
    this.activatedRoute.queryParams.subscribe((params) => {
      this.token = params.token;
    });
    this.resetForm = this.formBuilder.group({
      email: ['', Validators.required],
    });
  }

  // getter for form fields
  get f(): Record<string, AbstractControl> {
    return this.resetForm.controls;
  }

  public onSubmit(): void {
    this.submitted = true;

    // stop here if form is invalid
    if (this.resetForm.invalid) {
      return;
    }

    this.loading = true;
    this.authService.resetPassword(this.f.email.value).subscribe({
      next: () => {
        this.notificationService.showSuccess(
          'Un mail de confirmation de modification de votre mot de passe vous a été envoyé.'
        );
        this.router.navigate(['/login']);
      },
      complete: () => {
        this.loading = false;
      },
    });
  }

  public goLogin(): void {
    this.router.navigateByUrl('/login');
  }
}
