import { Injectable } from '@angular/core';
import { ActiveToast, ToastrService } from 'ngx-toastr';

@Injectable({
  providedIn: 'root',
})
export class NotificationService {
  constructor(private toastr: ToastrService) {}

  public showWarning(title: string, message = '', timeOut = 10000): ActiveToast<unknown> {
    return this.toastr.warning(message, title, {
      timeOut: timeOut,
      disableTimeOut: timeOut === 0,
    });
  }

  public showSuccess(title: string, message = '', timeOut = 10000): ActiveToast<unknown> {
    return this.toastr.success(message, title, {
      timeOut: timeOut,
      disableTimeOut: timeOut === 0,
    });
  }

  /**
   * By default, the error is displayed until clicked on.
   */
  public showError(title: string, message = '', timeOut = 0): ActiveToast<unknown> {
    return this.toastr.error(message, title, {
      timeOut: timeOut,
      disableTimeOut: timeOut === 0,
    });
  }

  public showInfo(title: string, message = '', timeOut = 10000): ActiveToast<unknown> {
    return this.toastr.info(message, title, {
      timeOut: timeOut,
      disableTimeOut: timeOut === 0,
    });
  }

  public showErrorPleaseRetry(title: string, timeOut = 0): void {
    this.showError(title, 'Merci de réessayer plus tard ou de contacter un administrateur', timeOut);
  }

  // Par défaut, l'erreur reste affichée jusqu'à ce qu'on clique dessus
  public showAppNewVersion(): void {
    const update = this.showInfo(
      'Nouvelle version',
      'Une nouvelle version est disponible, cliquer ici pour mettre à jour.',
      0
    );

    update.onTap.subscribe(() => {
      window.location.reload();
    });
  }
}
