import { CommonModule } from '@angular/common';
import type { Meta, StoryObj } from '@storybook/angular';
import { moduleMetadata } from '@storybook/angular';
import { SvgIconComponent } from '../../svg-icon/svg-icon.component';
import { ButtonTypeV3 } from './button-type.enum';
import { ButtonV3Component } from './button.component';

// More on how to set up stories at: https://storybook.js.org/docs/angular/writing-stories/introduction
const meta: Meta<ButtonV3Component> = {
  title: 'Components/Button',
  component: ButtonV3Component,
  tags: ['autodocs'],
  decorators: [
    moduleMetadata({
      declarations: [SvgIconComponent],
      imports: [CommonModule],
    }),
  ],
  argTypes: {},
};

export default meta;
type Story = StoryObj<ButtonV3Component>;

export const Primary: Story = {
  args: {
    label: 'Primary button',
    variant: ButtonTypeV3.Primary,
    size: 'medium',
    disabled: false,
  },
};

export const PrimarySmall: Story = {
  args: {
    ...Primary.args,
    size: 'small',
  },
};

export const PrimaryLarge: Story = {
  args: {
    ...Primary.args,
    size: 'large',
  },
};

export const PrimaryWithIcon: Story = {
  args: {
    ...Primary.args,
    label: 'Suivant',
    iconPosition: 'right',
    iconFolder: 'form',
    iconName: 'chevronRight',
  },
};

export const PrimaryWide: Story = {
  args: {
    ...Primary.args,
    wide: true,
  },
};

export const PrimaryBlack: Story = {
  args: {
    ...Primary.args,
    variant: ButtonTypeV3.PrimaryBlack,
  },
};

export const PrimaryWarning: Story = {
  args: {
    ...Primary.args,
    variant: ButtonTypeV3.PrimaryWarning,
  },
};

export const PrimarySuccess: Story = {
  args: {
    ...Primary.args,
    variant: ButtonTypeV3.PrimarySuccess,
  },
};

export const Secondary: Story = {
  args: {
    label: 'Secondary button',
    variant: ButtonTypeV3.Secondary,
  },
};

export const SecondaryDelete: Story = {
  args: {
    label: 'SecondaryDelete button',
    variant: ButtonTypeV3.SecondaryDelete,
  },
};

export const Tertiary: Story = {
  args: {
    label: 'Tertiary button',
    variant: ButtonTypeV3.Tertiary,
  },
};
