import { CommonModule } from '@angular/common';
import type { Meta, StoryObj } from '@storybook/angular';
import { moduleMetadata } from '@storybook/angular';
import { SvgIconComponent } from '../../svg-icon/svg-icon.component';
import { ButtonV3Component } from '../button/button.component';
import { ModalV3Component } from './modal.component';

// More on how to set up stories at: https://storybook.js.org/docs/angular/writing-stories/introduction
const meta: Meta<ModalV3Component> = {
  title: 'Components/Modal',
  component: ModalV3Component,
  tags: ['autodocs'],
  decorators: [
    moduleMetadata({
      declarations: [SvgIconComponent, ButtonV3Component],
      imports: [CommonModule],
    }),
  ],
  argTypes: {},
};

export default meta;
type Story = StoryObj<ModalV3Component>;

export const SingleButton: Story = {
  args: {
    opened: true,
    title: 'Title',
    content: 'Test text',
    validateLabel: 'Valider',
    cancelLabel: 'Annuler',
    singleButton: true,
  },
  render: (args: any) => ({
    template: `<app-v3-modal [opened]="${args.opened}"
    [validateLabel]="'${args.validateLabel}'"
    [cancelLabel]="'${args.cancelLabel}'"
    [title]="'${args.title}'"
    [singleButton]="${args.singleButton}"
    >
    ${args.content}
    </app-v3-modal>`,
  }),
};

export const TwoButtons: Story = {
  args: {
    ...SingleButton.args,
    singleButton: false,
  },
  render: SingleButton.render,
};

export const FormattedMessage: Story = {
  args: {
    ...SingleButton.args,
    singleButton: false,
    title: 'Travaillez-vous ici&nbsp;?',
    content:
      '<div class="modalContent"> \
        Un message sera envoyé aux membres de la structure : <br /> \
        <span class="emphasized">Le Son du Clic</span> \
      </div>',
  },
  render: SingleButton.render,
};

export const WithImage: Story = {
  args: {
    ...SingleButton.args,
    content:
      '<div> \
        <img src="../../../../../assets/img/resin-login.svg" alt="resin-login-image" /> \
        <h3>Additional text</h3> \
      </div>',
  },
  render: SingleButton.render,
};
