import { CommonModule } from '@angular/common';
import { EventEmitter } from '@angular/core';
import type { Meta, StoryObj } from '@storybook/angular';
import { moduleMetadata } from '@storybook/angular';
import { SvgIconComponent } from '../../../svg-icon/svg-icon.component';
import { TagItemV3Component } from './tag-item.component';

// More on how to set up stories at: https://storybook.js.org/docs/angular/writing-stories/introduction
const meta: Meta<TagItemV3Component> = {
  title: 'Components/Tag item',
  component: TagItemV3Component,
  tags: ['autodocs'],
  decorators: [
    moduleMetadata({
      declarations: [SvgIconComponent],
      imports: [CommonModule],
    }),
  ],
  argTypes: {},
};

export default meta;
type Story = StoryObj<TagItemV3Component>;

export const Small: Story = {
  args: {
    label: 'Label tag',
    size: 'small',

    action: new EventEmitter(),
  },
};

export const SmallWithIcon: Story = {
  args: {
    ...Small.args,
    iconFolder: 'ico',
    iconName: 'email',
  },
};

export const Medium: Story = {
  args: {
    label: 'Medium tag',
    color: 'white',
    size: 'medium',
  },
};

export const MediumSelected: Story = {
  args: {
    label: 'Medium tag',
    color: 'black',
    size: 'medium',
    iconFolder: 'ico',
    iconName: 'mail',
  },
};

export const MediumRed: Story = {
  args: {
    label: 'Médiathèque/Bibliothèque',
    color: 'red',
    size: 'medium',
    clickable: false,
  },
};

export const MediumBlack: Story = {
  args: {
    label: 'Médiathèque/Bibliothèque',
    color: 'black',
    size: 'medium',
  },
};

export const MediumWithIcon: Story = {
  args: {
    ...Medium.args,
    iconFolder: 'ico',
    iconName: 'email',
  },
};

export const Unclickable: Story = {
  args: {
    label: 'Unclickable tag',
    clickable: false,
  },
};

export const MultipleTags: Story = {
  render: (args) => ({
    props: args,
    template: `
    <div class="btn-grid">
      <app-v3-tag-item label="test 1" />
      <app-v3-tag-item label="test 2" />
      <app-v3-tag-item label="test 3" />
    </div>`,
  }),
};
