import { Component, EventEmitter, Input, Output } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Structure } from '../../../models/structure.model';
import { ProfileService } from '../../../profile/services/profile.service';
import { ButtonTypeV3 } from '../../../shared/components/v3/button/button-type.enum';

@Component({
  selector: 'app-card',
  templateUrl: './card.component.html',
  styleUrls: ['./card.component.scss'],
})
export class CardComponent {
  @Input() public structure: Structure;
  @Input() public isSelected: boolean;
  @Input() public isOrientation = false;
  @Input() public isOrientationRdv = false;
  @Input() public noDetails = false;
  @Input() public isInteractive = true;
  @Output() public showDetails: EventEmitter<Structure> = new EventEmitter<Structure>();
  @Output() public addToList: EventEmitter<Structure> = new EventEmitter<Structure>();
  @Output() public selectRDV: EventEmitter<Structure> = new EventEmitter<Structure>();
  @Output() public hover: EventEmitter<Structure> = new EventEmitter<Structure>();
  public buttonTypeEnumV3 = ButtonTypeV3;

  constructor(private route: ActivatedRoute, private router: Router, public profileService: ProfileService) {}

  /**
   * Display distance in m or km according to value
   */
  public formatDistance(): string {
    if (this.structure.distance > 1000) {
      return (this.structure.distance / 1000).toFixed(1).toString() + ' km';
    } else {
      return this.structure.distance + ' m';
    }
  }

  public cardClicked(): void {
    if (!this.noDetails) {
      this.showDetails.emit(this.structure);
      const queryString = this.route.snapshot.queryParamMap.get('search');
      this.router.navigate([], {
        relativeTo: this.route,
        queryParams: queryString
          ? {
              id: this.structure._id,
              search: queryString,
            }
          : {
              id: this.structure._id,
            },
      });
    }
  }

  public cardHover(): void {
    this.hover.emit(this.structure);
  }

  public cardAddToList(): void {
    this.addToList.emit(this.structure);
  }

  public cardRDV(): void {
    this.selectRDV.emit(this.structure);
  }
}
