import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { ButtonTypeV3 } from '../../../shared/components/v3/button/button-type.enum';
import { CategoryEnum } from '../../../shared/enum/category.enum';
import { TypeModal } from '../../enum/typeModal.enum';
import { Category } from '../../models/category.model';
import { Module } from '../../models/module.model';
import { SearchService } from '../../services/search.service';

@Component({
  selector: 'app-more-filters',
  templateUrl: './more-filters.component.html',
  styleUrls: ['./more-filters.component.scss'],
})
export class MoreFiltersComponent implements OnInit, OnChanges {
  constructor(public searchService: SearchService) {}

  @Input() public modalType: TypeModal;
  @Input() public categories: Category[];
  // checked modules filter list
  @Input() public modules: Module[] = [];
  @Output() searchEvent = new EventEmitter();
  @Output() closeEvent = new EventEmitter();
  public buttonTypeEnumV3 = ButtonTypeV3;
  public categoryEnum = CategoryEnum;

  // Checkbox variable
  public checkedModules: Module[] = [];

  ngOnInit(): void {
    // Manage checkbox
    this.checkedModules = this.modules.slice();
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.modalType) {
      this.checkedModules = this.modules.slice();
    }
  }

  // Management of the checkbox event (Check / Uncheck)
  public onCheckboxChange(event, category: string, text?: string): void {
    // Handle mouse or keyboard event
    const checkValue: string = event.target.value;
    const checked: boolean = event.target.checked;
    if (checked) {
      this.checkedModules.push(new Module(checkValue, category, text ? text : checkValue));
    } else {
      // Check if the module is present in the list and remove it
      if (this.searchService.getIndex(this.checkedModules, checkValue, category) > -1) {
        this.checkedModules.splice(this.searchService.getIndex(this.checkedModules, checkValue, category), 1);
      }
    }
  }
  // Clear only filters in the current modal
  public clearFilters(): void {
    this.categories.forEach((categ: Category) => {
      categ.modules.forEach((module: Module) => {
        const index = this.searchService.getIndex(this.checkedModules, module.id, categ.id);
        if (index > -1) {
          this.checkedModules.splice(index, 1);
        }
      });
    });
    this.emitModules(this.checkedModules);
  }

  // Sends an array containing all modules
  public emitModules(m: Module[]): void {
    this.searchEvent.emit(m);
  }

  public getModalType(): string {
    switch (this.modalType) {
      case TypeModal.accompaniment:
        return 'accompaniment';
      case TypeModal.training:
        return 'training';
      case TypeModal.public:
        return 'public';
      case TypeModal.equipments:
        return 'equipments';
      case TypeModal.moreFilters:
        return 'moreFilters';
      default:
        return '';
    }
  }

  public closeModal(): void {
    this.closeEvent.emit();
  }

  public handleCategoryCheckBox(event, category: Category): void {
    const modules = this.categories.filter((c: Category) => c.id === category.id)[0].modules;
    if (event.target.checked) {
      for (const module of modules) {
        if (this.searchService.getIndex(this.checkedModules, module.id, category.id) === -1) {
          this.checkedModules.push(new Module(module.id, category.id, module.displayText));
        }
      }
    } else {
      for (const module of modules) {
        if (this.searchService.getIndex(this.checkedModules, module.id, category.id) > -1) {
          this.checkedModules.splice(this.searchService.getIndex(this.checkedModules, module.id, category.id), 1);
        }
      }
    }
  }

  public getCategoryCheckboxStatus(c: Category): string {
    const selectedModule: Module[] = this.checkedModules.filter((m) => m.name === c.id);
    if (selectedModule.length === c.modules.length) {
      return 'checked';
    } else if (selectedModule.length === 0) {
      return 'unchecked';
    } else if (selectedModule.length && selectedModule.length > 0) {
      return 'halfChecked';
    }
  }
}
